#include <grape.h>


/* grape-methode "display"
 *  
 * Call              : GRAPE(triang2d,"display")()
 * 
 * Parameters        : TRIANG2D *triang2d
 *             
 * Description       : eine instance triang2d wird auf dem
 *                     standard-device ausgegeben als 
 *                     Grid oder Patch modell.
 *
 * Keywords          : display
 *
 * Author            : martin rumpf, konrad polthier
 *
 * Copyright (c) 1989     by Universitaet Bonn
 *                           Institut fuer Angewandte Mathematik
 *                           Sonderforschungsbereich 256
 *                           D-53115 Bonn
 */

static  TRIANG2D *t;
static  double *tx,*ty,*tz;
static  INT3 *vx,*nb;
static  GRAPHICDEVICE *dev;
static  void (*patch_normal)(),
    (*patch_vertex)(),
    (*begin_patch)(),
    (*end_patch)(),
    (*move)(),
    (*draw)();

/***********************************************/
/*                                             */
/*    hilfsvariablen zur normalenhandhabung    */
/*                                             */
/***********************************************/


#define ind(i) ((i)+3)%3

static int norm_size = 0;
static VEC3 *norm = NULL;

TRIANG2D *triang2d_display()
{
    int i,j,k,nt,np,first;
    VEC3 point,normal;

    t = (TRIANG2D *)START_METHOD(G_INSTANCE);
    ASSURE(t,
        "display : can't manage classes",
        END_METHOD(NULL));

    nt = t->number_of_elements;
    np = t->number_of_points;
    tx = t->x;
    ty = t->y;
    tz = t->z;
    vx = t->vertex;
    nb = t->neighbour;

    if(t->number_of_points == 0) END_METHOD(t);

    ASSURE((tx != NULL) && ( ty != NULL),
        "missing coordinates x or y",
        END_METHOD(NULL));

    /* graphik aufsetzen */

    dev = (GRAPHICDEVICE *)GRAPE(GraphicDevice,"get-stddev")();
   
    if(dev->grid_patch == G_GRID) {

	move = dev->move;
	draw = dev->draw;

	if(tz != NULL) {
	    for(j=0;j<nt;j++) 
              for(first = TRUE, i=0;i<3;i++)
                if((nb[j][i] < 0) || (nb[j][i] < j)) {
                  if(first) {    
                    point[0] = tx[vx[j][ind(i+1)]];
                    point[1] = ty[vx[j][ind(i+1)]];
	            point[2] = tz[vx[j][ind(i+1)]];
	            move(point);
                    first = FALSE;
                  }
                  point[0] = tx[vx[j][ind(i+2)]];
	          point[1] = ty[vx[j][ind(i+2)]];
	          point[2] = tz[vx[j][ind(i+2)]];
	          draw(point);
	        }
                else first = TRUE;
	    
	}
	else {
	    point[2] = 0.0;
	    for(j=0;j<nt;j++) 
              for(first = TRUE, i=0;i<3;i++)
                if((nb[j][i] < 0) || (nb[j][i] < j)) {
                  if(first) {    
                    point[0] = tx[vx[j][ind(i+1)]];
                    point[1] = ty[vx[j][ind(i+1)]];
	            move(point);
                    first = FALSE;
                  }
                  point[0] = tx[vx[j][ind(i+2)]];
	          point[1] = ty[vx[j][ind(i+2)]];
	          draw(point);
	        }
                else first = TRUE;
	  
	}

	END_METHOD(t);
    
    }

    patch_normal = dev->patch_normal;
    patch_vertex = dev->patch_vertex;
    begin_patch = dev->begin_patch;
    end_patch = dev->end_patch;

    /* durcharbeiten der Dreiecke */

    if(tz != NULL) {
	if(norm_size < np) {
	    if (norm_size != 0) vec3_free(norm,norm_size);
	    norm_size = np;
	    norm = vec3_alloc(norm_size);
	}

	for(j=0;j<np;j++) {
	    for(k=0;k<3;k++)
		norm[j][k] = 0.0;
	}

	for (i=0 ; i < nt; i++) {

	    /* normalenberechnung */

	    normal[0] = (ty[vx[i][1]]-ty[vx[i][0]]) * 
		        (tz[vx[i][2]]-tz[vx[i][0]])-
	                (tz[vx[i][1]]-tz[vx[i][0]]) * 
                        (ty[vx[i][2]]-ty[vx[i][0]]);
	    normal[1] = (tz[vx[i][1]]-tz[vx[i][0]]) * 
                        (tx[vx[i][2]]-tx[vx[i][0]])-
	                (tx[vx[i][1]]-tx[vx[i][0]]) * 
                        (tz[vx[i][2]]-tz[vx[i][0]]);
	    normal[2] = (tx[vx[i][1]]-tx[vx[i][0]]) * 
                        (ty[vx[i][2]]-ty[vx[i][0]])-
	        (ty[vx[i][1]]-ty[vx[i][0]]) * 
                (tx[vx[i][2]]-tx[vx[i][0]]);

	    g_vec3_normalize (normal);
	    for(k=0;k<3;k++) {
		for(j=0;j<3;j++)
		    norm[vx[i][k]][j] += normal[j];
	    }
	}

	for(j=0;j<np;j++) g_vec3_normalize(norm[j]);

	for(j=0;j<nt;j++)  {
	    begin_patch();
	    for(i=0;i<3;i++) {
		patch_normal(norm[vx[j][i]]);
		point[0] = tx[vx[j][i]];
		point[1] = ty[vx[j][i]];
		point[2] = tz[vx[j][i]];
		patch_vertex(point);
	    }
	    end_patch();
	}

    }
    else {
	normal[0] = 0.0;
	normal[1] = 0.0;
	normal[2] = 1.0;
	point[2] = 0.0;
	for(j=0;j<nt;j++)  {
	    begin_patch();
	    patch_normal(normal);
	    for(i=0;i<3;i++) {
		point[0] = tx[vx[j][i]];
		point[1] = ty[vx[j][i]];
		patch_vertex(point);
	    }
	    end_patch();
	}
    }

    END_METHOD(t);
}
