#include <grape.h>


/* grape-methode "shrink-disp"
 *  
 * Call              : GRAPE(triang3d,"shrink-disp")()
 *                     GRAPE(triang3d,"shrink-foc-tetra-disp")()
 *                     GRAPE(triang3d,"shrink-foc-point-disp")()
 *                     GRAPE(triang3d,"shrink-layer-disp")()
 *                     GRAPE(triang3d,"shrink-angle-disp")()
 *                     GRAPE(triang3d,"shrink")(p,f,fpar)
 * 
 * Parameters        : TRIANG3D *triang3d
 *                     double    p  shrinkparameter (0.0 - 1.0)
 *                     int (*f)()  funktion ob tetraeder 
 *                                 gezeichnet werden soll oder nicht
 *                                 f(triang3d,i,fpar)  i = index des tetraeders
 *
 * Description       : eine instance triang3d wird auf dem
 *                     standard-device ausgegeben als 
 *                     patch modell
 *                     (ohne segmentaufbau ).
 *                     falls f == NULL werden alle tetraeder dargestellt
 *                     layer = eine schicht um eine ebene
 *                     angle = alle tetras mit flaechenwinkeln zwischen
 *                             zwei werten <angle min> , <angle max>
 *                     focus-tetra = alle tetraeder in <nb width>  
 *                                   nachbarschaft  
 *                             eines tetraeders <index> 
 * Keywords          : shrink-disp
 *
 * Author            : martin rumpf
 *
 * Copyright (c) 1989     by Universitaet Bonn
 *                           Institut fuer Angewandte Mathematik
 *                           Sonderforschungsbereich 256
 *                           D-53115 Bonn
 */

#define EPSILON 1e-8

/* @SRC trng3d_dsply.c */

static  TRIANG3D *t;
static  double *tx,*ty,*tz;
static  double  a,b,c,d;
static  INT4 *vx,*nb;
static  GRAPHICDEVICE *dev;
static  void (*patch_normal)(),
             (*patch_vertex)(),
             (*begin_patch)(),
             (*end_patch)(),
             (*move)(),
             (*draw)();  

static VEC3 center;
static double par;

static void draw_bound(n,p0,p1,p2,p3)

     int n,p0,p1,p2,p3;

{

  int i,j;
  VEC3 v[4],normal;
  double norm;

  v[0][0] = tx[p0];v[0][1] = ty[p0];v[0][2] = tz[p0];
  v[1][0] = tx[p1];v[1][1] = ty[p1];v[1][2] = tz[p1];
  v[2][0] = tx[p2];v[2][1] = ty[p2];v[2][2] = tz[p2];
  v[3][0] = tx[p3];v[3][1] = ty[p3];v[3][2] = tz[p3];

  for(i=0;i<4;i++)
    for(j=0;j<3;j++)
      v[i][j] += par * (center[j] - v[i][j]);

  if (dev->grid_patch == G_GRID) {
    move(v[3]);
    for(i=1;i<=3;i++) 
      draw(v[i]);
    return;
  }   


  /* normalenberechnung */

  
  normal[0] = (v[2][1]-v[1][1])*(v[3][2]-v[1][2])-
              (v[2][2]-v[1][2])*(v[3][1]-v[1][1]);
  normal[1] = (v[2][2]-v[1][2])*(v[3][0]-v[1][0])-
              (v[2][0]-v[1][0])*(v[3][2]-v[1][2]);
  normal[2] = (v[2][0]-v[1][0])*(v[3][1]-v[1][1])-
              (v[2][1]-v[1][1])*(v[3][0]-v[1][0]);
  
  norm = sqrt(normal[0]*normal[0] +
	      normal[1]*normal[1] +
	      normal[2]*normal[2]  );
  if(norm > 1e-10) {
    normal[0] /= norm;
    normal[1] /= norm;
    normal[2] /= norm;
  }

  /* orientierung der normalen */

  if((normal[0]*(v[2][0]-v[0][0])+
      normal[1]*(v[2][1]-v[0][1])+
      normal[2]*(v[2][2]-v[0][2])) > 0) {
    normal[0] = - normal[0];
    normal[1] = - normal[1];
    normal[2] = - normal[2];
    begin_patch();
    patch_normal(normal);
    for(i=3;i>=1;i--)
      patch_vertex(v[i]);
    end_patch();
  }
  else {
    begin_patch();
    patch_normal(normal);
    for(i=1;i<=3;i++)
      patch_vertex(v[i]);
    end_patch();
  }

/*******************************************
  normal[0] = v[1][0] + 0.08 * normal[0];
  normal[1] = v[1][1] + 0.08 * normal[1];
  normal[2] = v[1][2] + 0.08 * normal[2];

  move(v[1]);
  draw(normal);
*******************************************/

}

static MANAGER *mgr;
extern double g_maintime;

/**********************************/
/*     shrink-disp                */
/**********************************/

TRIANG3D *triang3d_shrink_disp()

{ 

  int i,j,nt,np;
  static SPLINE1D *sp_0;
  static FUNCTION_RULER *rul_0 = NULL;

  t = (TRIANG3D *)START_METHOD(G_INSTANCE);
  ASSURE(t,
	 "shrink-disp : can't manage classes",
	 END_METHOD(NULL));

  if(rul_0 == NULL) {
    sp_0 = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("shrink-disp");
    GRAPE(sp_0,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    rul_0 = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_0,&g_maintime,
                           &par,"shrink-disp",RULER_DOUBLE);
  }

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();  
  if(GRAPE(mgr,"new-handle")(triang3d_shrink_disp,1))
    GRAPE(mgr,"add-inter")(rul_0);

  GRAPE(t,"shrink")(par,NULL,NULL);

  END_METHOD(t);

}

/**********************************/
/*     shrink-foc-tetra-disp    */
/**********************************/

  
typedef struct focus_tetra_parm_type {int i,nbw;} FOCUS_TETRA_PARM;
static FOCUS_TETRA_PARM focus_tetra = { 0,0 };

static int tetra_search(t,i,fi,d)

TRIANG3D *t;
int i,fi,d;

{  int k,l;

   if(fi==i) return(1);
   if(d==0) return(0);
   for(k=0;k<4;k++)
     if((l=t->neighbour[i][k]) >= 0)
       if(tetra_search(t,l,fi,d-1)) return(1);
   return(0);
}

static int f_focus_tetra(t,i,focus_tetra)

TRIANG3D *t;
int i;
FOCUS_TETRA_PARM *focus_tetra;

{
   if(focus_tetra->nbw < 0) 
     return(tetra_search(t,i,focus_tetra->i,0));
   if(focus_tetra->nbw > 3) 
     return(tetra_search(t,i,focus_tetra->i,3));
   return(tetra_search(t,i,focus_tetra->i,focus_tetra->nbw));
}


TRIANG3D *triang3d_shrink_foc_tetra_disp()

{ 

  int i,j,nt,np;
  static SPLINE1D *sp_0;
  static FUNCTION_RULER *rul_0 = NULL;
  static RULER *rul_i = NULL;
  static RULER *rul_nbw = NULL;

  t = (TRIANG3D *)START_METHOD(G_INSTANCE);
  ASSURE(t,
	 "shrink-disp : can't manage classes",
	 END_METHOD(NULL));

  if(rul_0 == NULL) {
    sp_0 = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("shrink-parm");
    GRAPE(sp_0,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    rul_0 = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_0,&g_maintime,
                           &par,"shrink-parm",RULER_DOUBLE);
    rul_i = (RULER *)GRAPE(Ruler,"new-instance")
                          (&focus_tetra.i,"tetra index",RULER_INTEGER);
    rul_nbw = (RULER *)GRAPE(Ruler,"new-instance")
                            (&focus_tetra.nbw,"nb width",RULER_INTEGER);
  }

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();  
  if(GRAPE(mgr,"new-handle")(triang3d_shrink_disp,3)) {
    GRAPE(mgr,"add-inter")(rul_0);
    GRAPE(mgr,"add-inter")(rul_i);
    GRAPE(mgr,"add-inter")(rul_nbw);
  }

  GRAPE(t,"shrink")(par,f_focus_tetra,&focus_tetra);

  END_METHOD(t);

}

/**********************************/
/*     shrink-foc-point-disp    */
/**********************************/

  
typedef struct focus_point_parm_type {int i,nbw;} FOCUS_POINT_PARM;
static FOCUS_POINT_PARM focus_point = { 0,0 };

static int point_search(t,i,fi,d)

TRIANG3D *t;
int i,fi,d;

{  int k,l;

   for(k=0;k<4;k++)
     if(t->vertex[i][k] == fi) return(1);
   if(d==0) return(0);
   for(k=0;k<4;k++)
     if((l=t->neighbour[i][k]) >= 0)
       if(point_search(t,l,fi,d-1)) return(1);
   return(0);
}

static int f_focus_point(t,i,focus_point)

TRIANG3D *t;
int i;
FOCUS_POINT_PARM *focus_point;

{
   if(focus_point->nbw < 0) 
     return(point_search(t,i,focus_point->i,0));
   if(focus_point->nbw > 3) 
     return(point_search(t,i,focus_point->i,3));
   return(point_search(t,i,focus_point->i,focus_point->nbw));
}


TRIANG3D *triang3d_shrink_foc_point_disp()

{ 

  int i,j,nt,np;
  static SPLINE1D *sp_0;
  static FUNCTION_RULER *rul_0 = NULL;
  static RULER *rul_i = NULL;
  static RULER *rul_nbw = NULL;

  t = (TRIANG3D *)START_METHOD(G_INSTANCE);
  ASSURE(t,
	 "shrink-disp : can't manage classes",
	 END_METHOD(NULL));

  if(rul_0 == NULL) {
    sp_0 = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("shrink-parm");
    GRAPE(sp_0,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    rul_0 = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_0,&g_maintime,
                           &par,"shrink-parm",RULER_DOUBLE);
    rul_i = (RULER *)GRAPE(Ruler,"new-instance")
                          (&focus_point.i,"point index",RULER_INTEGER);
    rul_nbw = (RULER *)GRAPE(Ruler,"new-instance")
                            (&focus_point.nbw,"nb width",RULER_INTEGER);
  }

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();  
  if(GRAPE(mgr,"new-handle")(triang3d_shrink_disp,3)) {
    GRAPE(mgr,"add-inter")(rul_0);
    GRAPE(mgr,"add-inter")(rul_i);
    GRAPE(mgr,"add-inter")(rul_nbw);
  }

  GRAPE(t,"shrink")(par,f_focus_point,&focus_point);

  END_METHOD(t);

}

/**********************************/
/*     shrink-layer-disp          */
/**********************************/

typedef struct layer_parm_type {double eps,d,nx,ny,nz;} LAYER_PARM;
static LAYER_PARM layer = { 1.0,0.0,0.0,0.0,1.0};

static int f_layer(t,i,l)

TRIANG3D *t;
int i;
LAYER_PARM *l;

{
   int j;
   double center[3];
   double h;

   for(center[0] = center[1] = center[2] = j =0 ;j < 4; j++) {
     center[0] += 0.25 * t->x[t->vertex[i][j]];
     center[1] += 0.25 * t->y[t->vertex[i][j]];
     center[2] += 0.25 * t->z[t->vertex[i][j]];
   }

   h = center[0] * l->nx +
       center[1] * l->ny +
       center[2] * l->nz - l->d;

   return(fabs(h) < l->eps);


}


TRIANG3D *triang3d_shrink_layer_disp()

{ 

  int i,j,nt,np;
  static SPLINE1D *sp_0,*sp_e,*sp_d,*sp_nx,*sp_ny,*sp_nz;
  static FUNCTION_RULER *rul_0 = NULL;
  static FUNCTION_RULER *rul_e = NULL;
  static FUNCTION_RULER *rul_d = NULL;
  static FUNCTION_RULER *rul_nx = NULL;
  static FUNCTION_RULER *rul_ny = NULL;
  static FUNCTION_RULER *rul_nz = NULL;

  t = (TRIANG3D *)START_METHOD(G_INSTANCE);
  ASSURE(t,
	 "shrink-disp : can't manage classes",
	 END_METHOD(NULL));

  if(rul_0 == NULL) {
    sp_0 = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("shrink-parm");
    GRAPE(sp_0,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    sp_e = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("layer width");
    GRAPE(sp_e,"set-linear")(1.0,1.0,0.0,1.0,3,10);
    sp_d = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("layer d");
    GRAPE(sp_d,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    sp_nx = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("normal x");
    GRAPE(sp_nx,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    sp_ny = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("normal y");
    GRAPE(sp_ny,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    sp_nz = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("normal z");
    GRAPE(sp_nz,"set-linear")(1.0,1.0,0.0,1.0,3,10);
    
    rul_0 = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_0,&g_maintime,
                           &par,"shrink-parm",RULER_DOUBLE);
    rul_e = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_e,&g_maintime,
                           &layer.eps,"layer width",RULER_DOUBLE);
    rul_d = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_d,&g_maintime,
                           &layer.d,"layer d",RULER_DOUBLE);
    rul_nx= (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_nx,&g_maintime,
                           &layer.nx,"normal x",RULER_DOUBLE);
    rul_ny= (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_ny,&g_maintime,
                           &layer.ny,"normal y",RULER_DOUBLE);
    rul_nz= (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_nz,&g_maintime,
                           &layer.nz,"normal z",RULER_DOUBLE);
    }

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();  
  if(GRAPE(mgr,"new-handle")(triang3d_shrink_disp,6)) {
    GRAPE(mgr,"add-inter")(rul_0);
    GRAPE(mgr,"add-inter")(rul_e);
    GRAPE(mgr,"add-inter")(rul_d);
    GRAPE(mgr,"add-inter")(rul_nx);
    GRAPE(mgr,"add-inter")(rul_ny);
    GRAPE(mgr,"add-inter")(rul_nz);
  }

  GRAPE(t,"shrink")(par,f_layer,&layer);

  END_METHOD(t);

}

/**********************************/
/*     shrink-angle-disp          */
/**********************************/

typedef struct angle_parm_type {double min,max;} ANGLE_PARM;
static ANGLE_PARM angle = { 0.0,1.5707963};

#define scpr(i,j,k) (t->x[(j)]-t->x[(i)])*(t->x[(k)]-t->x[(i)]) + \
                    (t->y[(j)]-t->y[(i)])*(t->y[(k)]-t->y[(i)]) + \
                    (t->z[(j)]-t->z[(i)])*(t->z[(k)]-t->z[(i)]) 
                  
static double calc_angle(t,i,j,k)

TRIANG3D *t;
int i,j,k;

{

  double nj,nk,sp;

  nj = sqrt(scpr(i,j,j));
  nk = sqrt(scpr(i,k,k));
  sp = fabs(scpr(i,j,k));
  
  if((nj > EPSILON) && (nk > EPSILON))
    return(acos(sp/nj/nk));
  return(0.0);
}

#undef scpr 


static int f_angle(t,it,a)

TRIANG3D *t;
int it;
ANGLE_PARM *a;

{
   int i,j,k;
   double lmin;
   double alpha;

   lmin = 2.0;

   for(i=0;i<4;i++)
     for(j=0;j<4;j++)
       if(i!=j)
	 for(k=0;k<4;k++)
	   if((k!=i) && (k!=j)) {
	     alpha = calc_angle(t,
				t->vertex[it][i],
				t->vertex[it][j],
				t->vertex[it][k]);
	     if(alpha<lmin) lmin = alpha;
	   }
   return((lmin>a->min) && (lmin<a->max));

}

TRIANG3D *triang3d_shrink_angle_disp()

{ 

  int i,j,nt,np;
  static SPLINE1D *sp_0,*sp_min,*sp_max;
  static FUNCTION_RULER *rul_0 = NULL;
  static FUNCTION_RULER *rulamin = NULL;
  static FUNCTION_RULER *rulamax = NULL;

  t = (TRIANG3D *)START_METHOD(G_INSTANCE);
  ASSURE(t,
	 "shrink-disp : can't manage classes",
	 END_METHOD(NULL));

  if(rul_0 == NULL) {
    sp_0 = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("shrink-parm");
    GRAPE(sp_0,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    sp_min= (SPLINE1D *)GRAPE(Spline1d,"new-instance")("angle min");
    GRAPE(sp_min,"set-linear")(0.0,0.0,0.0,1.0,3,10);
    sp_max = (SPLINE1D *)GRAPE(Spline1d,"new-instance")("angle max");
    GRAPE(sp_max,"set-linear")(1.5707963,1.5707963,0.0,1.0,3,10);
   
    rul_0 = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                          (sp_0,&g_maintime,
                           &par,"shrink-parm",RULER_DOUBLE);
    rulamin = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                            (sp_min,&g_maintime,
                             &angle.min,"angle min",RULER_DOUBLE);
    rulamax = (FUNCTION_RULER *)GRAPE(Function_Ruler,"new-instance")
                            (sp_max,&g_maintime,
                             &angle.max,"angle max",RULER_DOUBLE);
  }

  mgr = (MANAGER *)GRAPE(Manager,"get-stdmgr")();  
  if(GRAPE(mgr,"new-handle")(triang3d_shrink_disp,3)) {
    GRAPE(mgr,"add-inter")(rul_0);
    GRAPE(mgr,"add-inter")(rulamin);
    GRAPE(mgr,"add-inter")(rulamax);
  }

  GRAPE(t,"shrink")(par,f_angle,&angle);

  END_METHOD(t);

}

/**********************************/
/*     eigentliches shrink        */
/**********************************/
  
TRIANG3D *triang3d_shrink(p,f,fpar)

 double p;
 int (*f)();
 char *fpar;
 
{ 

  int i,j,nt,np;

  par = p;

  t = (TRIANG3D *)START_METHOD(G_INSTANCE);
  ASSURE(t,
	 "shrink : can't manage classes",
	 END_METHOD(NULL));
  
  nt = t->number_of_elements;
  np = t->number_of_points;
  tx = t->x;
  ty = t->y;
  tz = t->z;
  vx = t->vertex;
  nb = t->neighbour;
 
  /* graphik aufsetzen */

  dev = (GRAPHICDEVICE *)GRAPE(GraphicDevice,"get-stddev")(); 

  patch_normal = dev->patch_normal;
  patch_vertex = dev->patch_vertex;
  begin_patch = dev->begin_patch;
  end_patch = dev->end_patch;
  move = dev->move;
  draw = dev->draw;
  
  /* durcharbeiten der tetraeder */
  
  for (i=0 ; i < nt; i++) 
    if((f == NULL) || f(t,i,fpar)) {
    for(center[0] = center[1] = center[2] = j =0 ;j < 4; j++) {
      center[0] += 0.25 * tx[vx[i][j]];
      center[1] += 0.25 * ty[vx[i][j]];
      center[2] += 0.25 * tz[vx[i][j]];
    }
    for(j=0 ;j < 4; j++) 
      switch(j) {
      case 0 : 
	draw_bound(nb[i][j],vx[i][0],vx[i][1],vx[i][2],vx[i][3]);
	break;
      case 1 : 
	draw_bound(nb[i][j],vx[i][1],vx[i][0],vx[i][2],vx[i][3]);
	break;
      case 2 : 
	draw_bound(nb[i][j],vx[i][2],vx[i][0],vx[i][1],vx[i][3]);
	break;
      case 3 : 
	draw_bound(nb[i][j],vx[i][3],vx[i][0],vx[i][1],vx[i][2]);
	break;
      }
  }

  END_METHOD(t);
}
