#include <grape.h>


/* grape-methode "display","universal"
 *  
 * Call              : GRAPE(timescene,"universal")(method)
 *                     GRAPE(timescene,"display")()
 * 
 * Parameters        : TIMESCENE *timescene
 *                     char      *method     methode die universal weiterreicht
 * Description       : die display aehnliche universaloperation 
 *                     wird ausgefuehrt . das heisst man kann argumentfreie 
 *                     methoden einer Timescene schicken wenn sie nur auf allen
 *                     blaettern des baums definiert sind.
 *                     das aktuelle object wird gegebenenfalls neu ermittelt
 *                     ( get-object an timescene->dynamic)
 *
 * Keywords          : universal
 *
 * Author            : martin rumpf
 *
 * Copyright (c) 1992     by Universitaet Bonn
 *                           Institut fuer Angewandte Mathematik
 *                           Sonderforschungsbereich 256
 *                           D-53115 Bonn
 *
 */



TIMESCENE *timescene_universal(char *method)
{ 
  TIMESCENE *self;
  INSTANCE *inst;
  MATRIX44 help;
  GRAPHICDEVICE *dev;
  SUPROP *suprop;

  self = (TIMESCENE *)START_METHOD(G_INSTANCE);
  ASSURE (self, "", END_METHOD (NULL));
 
  if((self->time != self->object_time) || (self->object == NULL)) {
    ASSURE((self->dynamic != NULL),
	   "universal : timescene contains no dynamic instance",
	   END_METHOD(NULL));
    inst = (INSTANCE *)GRAPE(self->dynamic,"get-object")(self->object,self->time);
    if(inst != NULL) {
      self->object = inst;
      self->object_time = self->time;
    }
    else 
      self->time = self->object_time;
  }
  
  if(!g_strempty (self->dynamic_method_name) &&
     (strcmp(self->dynamic_method_name,"no-display"))) {
    if(self->matrix_flag != G_MATRIX_NONE) {
      dev = (GRAPHICDEVICE *)GRAPE(GraphicDevice,"get-stddev")();
      dev->transform(G_MODE_GET,self->matrix_flag,help);
      dev->transform(G_MODE_PRE,self->matrix_flag,self->object_trans);
    }     
    if(self->suprop) {
      suprop = (SUPROP *)GRAPE (Suprop, "new-instance")("scene disp suprop");
      /* hope that's ok. */
      suprop->enabled = self->suprop->enabled;
      GRAPE (suprop, "store")();
      GRAPE (self->suprop,"load")();
    }
    GRAPE(self->dynamic,self->dynamic_method_name)();
    if(self->matrix_flag != G_MATRIX_NONE)
      dev->transform(G_MODE_SET,self->matrix_flag,help);
    if(self->suprop) {
      GRAPE (suprop, "load")();
      GRAPE (suprop, "delete")();
    }
  }
  
  GRAPE(self,"^universal")(method);
     
  END_METHOD(self);
}


TIMESCENE *timescene_display()
{ 
  TIMESCENE *self;
  INSTANCE *inst;
  MATRIX44 help;
  GRAPHICDEVICE *dev;
  SUPROP *suprop;

  self = (TIMESCENE *)START_METHOD(G_INSTANCE);
  
  if((self->time != self->object_time) || (self->object == NULL)) {
    ASSURE((self->dynamic != NULL),
	   "display : timescene contains no dynamic instance",
	   END_METHOD(NULL));
    inst = (INSTANCE *)GRAPE(self->dynamic,"get-object")
      (self->object,self->time);
    if(inst != NULL) {
      self->object = inst;
      self->object_time = self->time;
    }
    else 
      self->time = self->object_time;
  }

  if(!g_strempty(self->dynamic_method_name) &&
     (strcmp(self->dynamic_method_name,"no-display"))) {

    if(self->matrix_flag != G_MATRIX_NONE) {
      dev = (GRAPHICDEVICE *)GRAPE(GraphicDevice,"get-stddev")();
      dev->transform(G_MODE_GET,self->matrix_flag,help);
      dev->transform(G_MODE_PRE,self->matrix_flag,self->object_trans);
    }   

    if(self->suprop) {
      suprop = (SUPROP *)GRAPE (Suprop, "new-instance")
	("timescene disp suprop");
      suprop->enabled = self->suprop->enabled;
      GRAPE (suprop, "store")();
      GRAPE (self->suprop,"load")();
    }

    GRAPE(self->dynamic,self->dynamic_method_name)();

    if(self->suprop) {
      GRAPE (suprop, "load")();
      GRAPE (suprop, "delete")();
    }
    if(self->matrix_flag != G_MATRIX_NONE)
      dev->transform(G_MODE_SET,self->matrix_flag,help);
  }
  
  GRAPE(self,"^display")();
     
  END_METHOD(self);
}
