#include <grape.h>


/* grape-methode "get-object"
 *  
 * Call              : INSTANCE *GRAPE(timestep,"get-object")(inst,time)
 *                  
 * Parameters        : TIMESTEP *timestep
 *                     INSTANCE *inst        zu aktualisierendes object
 *                     double time            aktuelle zeit
 * Description       : ermittelt das aktuelle object zur zeit time
 *                     und beschreibt inst damit oder gibt ein neues zurueck
 *                     (wichtig falls inst = NULL)
 *
 * Keywords          : get-object
 *
 * Author            : martin rumpf
 *
 * Copyright (c) 1992     by Universitaet Bonn
 *                           Institut fuer Angewandte Mathematik
 *                           Sonderforschungsbereich 256
 *                           D-53115 Bonn
 *
 *
 */

INSTANCE *timestep_get_object(INSTANCE *inst, double t)
{
  double mu;
  TIMESTEP *s;

  s = (TIMESTEP *)START_METHOD(G_INSTANCE);
  ASSURE(s,"get-object : can't handle classes\n",END_METHOD(NULL));
  ASSURE((s->pre_object != NULL) || (s->post_object != NULL),
	 "get-object : no pre_object and no post_object found",
	 END_METHOD(NULL));
  if(inst == NULL){
    if(s->post_object != NULL)
      inst = (INSTANCE *)GRAPE(s->post_object->class,"new-instance")
	                      ("interpol");
    else
      inst = (INSTANCE *)GRAPE(s->pre_object->class,"new-instance")
	                      ("interpol");
  }
  do {
    if(s == NULL) END_METHOD(NULL);
    if(fabs (t - s->time) < ZERO_MACHINE) {
      if(s->post_object != NULL)
	GRAPE(inst,"interpol")(s->post_object,NULL,0.0);
      else
	GRAPE(inst,"interpol")(s->pre_object,NULL,0.0);
      END_METHOD(inst);
    }
    if(t < s->time) {
      /* check for zycle */
      if((s->pre_step != NULL) && (s->pre_step->time < s->time))
	  s = s->pre_step;
      else {
	if(s->pre_object != NULL)
	  GRAPE(inst,"interpol")(s->pre_object,NULL,0.0);
	else
	  GRAPE(inst,"interpol")(s->post_object,NULL,0.0);
	END_METHOD(inst);
      }
    }
    else { 
      if((s->post_step != NULL) &&
	 (s->time < s->post_step->time)) {
	if(t >= s->post_step->time)
	  s = s->post_step;
	else {
	  ASSURE(s->time != s->post_step->time,
		 "get-object : two timesteps with an identical time",
		 END_METHOD(NULL));
	  mu = (t - s->time)/(s->post_step->time - s->time);
	  GRAPE(inst,"interpol")(s->post_object,s->post_step->pre_object,mu);
	  END_METHOD(inst);
	}
      }
      else {
	if(s->post_object != NULL)
	  GRAPE(inst,"interpol")(s->post_object,NULL,0.0);
	else
	  GRAPE(inst,"interpol")(s->pre_object,NULL,0.0);
	END_METHOD(inst);
      }
    }
  } while(1);

}
