/*
 * main program for "rotate"
 *
 * Description       : Create the new class, its method, the editor and
 *                     a Rot2d instance which is then handled.
 *
 * Author            : Bernd Oberknapp
 *
 * Copyright (c) 1995  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 *
 * $Log:$
 */

#include <grape.h>
#include "rot2d.h"


/*
 * Define the new class pointer, it is initialized below
 */
CLASS *Rot2d = NULL;


int main()
{
  MANAGER *mgr;
  ROT2D *rot2d;

  /*
   * First we call "new-class" to create our class Rot2d. The method
   * is send to the superclass, its parameters are the name and the
   * size of the structure, this size will be used by "new-instance"
   * to allocate memory for the instance.
   */
  Rot2d = (CLASS *)GRAPE(Triang2d, "new-class")("Rot2d", sizeof(ROT2D));
  ASSURE(Rot2d, "main: can't create class Rot2d", return 5);

  /*
   * Now add the methods for the new class and the two methods
   * we have written for the existing class Triang1d. "add-method"
   * is called on the class the method is defined on, parameters
   * are the method name and the pointer to the function. This
   * pointer is used when GRAPE()() is called to call the function
   * corresponding to the method name.
   */
  GRAPE(Rot2d, "add-method")("new-instance", rot2d_new_instance);
  GRAPE(Rot2d, "add-method")("free", rot2d_free);
  GRAPE(Rot2d, "add-method")("hardcopy", rot2d_hardcopy);
  GRAPE(Rot2d, "add-method")("softcopy", rot2d_softcopy);
  GRAPE(Rot2d, "add-method")("rotate-send", rot2d_rotate_send);
  GRAPE(Rot2d, "add-method")("display", rot2d_display);
  GRAPE(Rot2d, "add-method")("get-info", rot2d_get_info);
  GRAPE(Rot2d, "add-method")("follow", rot2d_follow);
  GRAPE(Rot2d, "add-method")("get-editor", rot2d_get_editor);
  GRAPE(Rot2d, "add-method")("update-point", rot2d_update_point);
  GRAPE(Triang1d, "add-method")("insert-point", triang1d_insert_point);
  GRAPE(Triang1d, "add-method")("delete-point", triang1d_delete_point);
  GRAPE(Rot2d, "add-method")("insert-point", rot2d_insert_point);
  GRAPE(Rot2d, "add-method")("delete-point", rot2d_delete_point);

  /*
   * Now we can create our first Rot2d instance, it is called "rot".
   * This could fail so we test the return value.
   */
  rot2d = (ROT2D *)GRAPE(Rot2d, "new-instance")("rot");
  ASSURE(rot2d, "main: can't create object", return 5);

  /*
   * Now we bind the rot2d variables to the rulers and
   * create the buttons that will send our methods.
   */
  ASSURE(GRAPE(rot2d, "get-editor")(), "main: can't create editor", return 5);

  /*
   * Since we want to work with our Rot2d instance interactively
   * we have to start the manager. This is done by calling "handle"
   * with the instance as parameter. The control will be returned
   * to this function when the exit button is pressed.
   */
  mgr = (MANAGER *)GRAPE(Manager, "get-stdmgr")();
  GRAPE(mgr, "handle")(rot2d);

  return 0;
}
