/*
 * grape-module "rot2d_xdr"
 *
 * Description       : Archiving method for class Rot2d. This method will
 *                     be used both for writing and reading, it uses the
 *                     GRAPE XDR utility functions to write the instance
 *                     variables. See the section about the GRAPE XDR
 *                     archiving system for more information.
 *
 * Author            : Bernd Oberknapp
 *
 * Copyright (c) 1996  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 *
 * $Log:$
 */

#include <grape.h>
#include "rot2d.h"


ROT2D *rot2d_xdr(XDR *xdrp)
{
  ROT2D *self;
  /*
   * The version variable allows to change the Rot2d structure and still
   * be able to read old versions. If we add an integer flag to the
   * structure we just have to set ver to two and add the block
   *
   * if(ver > 1)
   *   if(!xdr_int(xdrp, &self->flag))
   *     END_METHOD(NULL);
   *
   * before the END_METHOD(self);. If a old version is read, ver will be
   * set to one and this new block will be skipped (it might be necessary
   * to initialize the flag in this case instead).
   */
  int ver = 1;

  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * Write or read version, superclass and Rot2d instance variable.
   * We have to pass pointer to version and instance variables to the
   * xdr functions to be able to change them on reading. If writing
   * of reading fails the method is aborted immediately.
   */
  if(!g_xdr_version(xdrp, &ver) ||
     !g_xdr_superclass(xdrp, (INSTANCE *)self) ||
     !g_xdr_instance(xdrp, (INSTANCE **)&self->curve) ||
     !g_xdr_instance(xdrp, (INSTANCE **)&self->axis) ||
     !  xdr_double(xdrp, &self->angle) ||
     !  xdr_int(xdrp, &self->discr) ||
     !  xdr_int(xdrp, &self->current))
    END_METHOD(NULL);

  END_METHOD(self);
}
